/**
* ManualThresholding_.java
* Created on 30 August 2005, 19:23 by Manuel Guillermo Forero-Vargas
* e-mail: mgforero@yahoo.es
*
* Manual thresholding algorithm
* Function: This plugin binarises the image by the threshold chosen by the user
*
* This plugin do not create a new image.
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;

public class ManualThresholding_ implements PlugInFilter
{
    ImagePlus imp;
    protected   int threshold=128;
    
    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        //----------------------------------------------------------------------
        //Creates a dialog box, allowing the user to enter the requested threshold
        GenericDialog gd=new GenericDialog("Manual threshold",IJ.getInstance());
        gd.addNumericField("Threshold:",threshold,0,3, "");
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        threshold=(int)gd.getNextNumber();
        this.imp=imp;
        //----------------------------------------------------------------------
         return IJ.setupDialog(imp,DOES_8G+DOES_8C+DOES_16);
    }

    public void run(ImageProcessor ip)
    {
        ip.threshold(threshold);
        ip.setMinAndMax(0,255);
        imp.setDisplayRange(0,255);
    }

    void showAbout()
    {
        IJ.showMessage("About ManualThresholding_...",
        "This plug-in binarize an 8-bit image by using a manual threshold.");
    }
}
