/**
 * Spherical_cells_detection.java
 * Created on 27th February 2018, 9:51
 * This plug-in filter obtains the spherical cells in a stack
 *
 * This plugin works in 2D and does not create a new stack.
 *
 * Input: Algorithm is applied to a 8-bit grey level stack.
 *
 * Output: Spherical cells stack
 * 
 * Author: Manuel Guillermo Forero-Vargas and Cristian Saúl Gonzalez
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.plugin.filter.*;

public class Spherical_cells_detection implements PlugInFilter
{
    int radius=10,thickMem=2;
    ImagePlus imp;

    @Override
    public int setup(String arg, ImagePlus imp)
    {
        this.imp=imp;
        GenericDialog gd= new GenericDialog("Spherical_cell_detection");
        gd.addNumericField("Radius:",radius,0);
        gd.addNumericField("Membrane thickness:",thickMem,0);
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        radius=(int)gd.getNextNumber();
        thickMem=(int)gd.getNextNumber();
        return DOES_8G+STACK_REQUIRED;
    }

    @Override
    public void run(ImageProcessor ip)
    {
        int r2=radius*radius;
        int rAux=radius+thickMem;
        int rAux2=rAux*rAux;
        int deep=imp.getNSlices();
        int height=imp.getHeight();
        int width=imp.getWidth();
        ImageStack  stack=imp.getStack();
        boolean bOut;//Flag indicating if a whole circle is included inside a white region
        int i,ii,j,jj,k,kk;
        int[][][] pixelsO=new int[deep][width][height];
        int[][][] output=new int[deep][width][height];
        int a,c;

        //Creates a copy of the original stack
        for(k=0;k<deep;k++)
        {
            byte[] pixels=(byte[])stack.getProcessor(k+1).getPixels();
            for(c=j=0;j<height;j++)
                for(i=0;i<width;i++)
                    pixelsO[k][i][j]=pixels[c++]&0xff;
        }
        //------------------------------------------------------------------ 
        for(k=radius;k<deep-radius;k++)
            for(j=radius;j<height-radius;j++)
                for(i=radius;i<width-radius;i++)
                    if(pixelsO[k][i][j]==255)
                    {
                        for(kk=k-radius,bOut=true;kk<=k+radius;kk++)
                        {
                            a=(kk-k)*(kk-k);
                            for (jj=j-radius;jj<=j+radius;jj++)
                            {
                                c=(jj-j)*(jj-j)+a;
                                for (ii=i-radius;ii<=i+radius;ii++)
                                    if(c+(ii-i)*(ii-i)<=r2&&pixelsO[kk][ii][jj]==0)
                                    {//If the region is not fully white
                                        bOut=false;
                                        break;
                                    }
                                if(bOut==false)
                                    break;
                            }
                            if(bOut==false)
                                break;
                        }
                        if(bOut==true)//If the region is bigger than the specified region, it is deleted
                            for(kk=k-rAux;kk<=k+rAux;kk++)
                            {
                                a=(kk-k)*(kk-k);
                                if(kk>=0&&kk<deep)
                                    for (jj=j-rAux;jj<=j+rAux;jj++)
                                    {
                                        c=(jj-j)*(jj-j)+a;
                                        if(jj>=0&&jj<height)
                                            for (ii=i-rAux;ii<=i+rAux;ii++)
                                            if(ii>=0&&ii<width)
                                            {
                                                if(c+(ii-i)*(ii-i)<=r2&&pixelsO[kk][ii][jj]==255)
                                                    output[kk][ii][jj]=255;
                                                if(c+(ii-i)*(ii-i)<=rAux2&&pixelsO[kk][ii][jj]==255)
                                                    pixelsO[kk][ii][jj]=0;
                                            }
                                    }
                            }
                    }
        for(k=0;k<deep;k++)
        {
            byte[] pixels=(byte[])stack.getProcessor(k+1).getPixels();
            for(c=j=0;j<height;j++)
                for(i=0;i<width;i++)
                    pixels[c++]=(byte)pixelsO[k][i][j];
        }
        imp.updateAndDraw();
        ImageStack stack1=new ImageStack(width,height);
        for(k=0;k<deep;k++)
        {
            ByteProcessor ip1=new ByteProcessor(width,height); 
            byte[] pixels1=(byte[])ip1.getPixels();
            for(c=j=0;j<height;j++)
                for(i=0;i<width;i++)
                    pixels1[c++]=(byte)output[k][i][j];
            stack1.addSlice("",ip1);
        }
        ImagePlus imp1=new ImagePlus("Spheres",stack1);
        imp1.show();
    }
}
