/**
* VolumeFilter_.java
* Created on 24th July 2006, 8:52 by Manuel Guillermo Forero-Vargas
* e-mail: mgforero@yahoo.es
*
* Volume filter algorithm
* Function: This plug-in eliminates the 3D-objects that are smaller or bigger than
* a threshold volume defined by the user.
* 
* This plugin do not create a new stack.
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.process.ImageProcessor;
import ij.process.ShortProcessor;
import ij.process.StackStatistics;
import ij.gui.GenericDialog;
import ij.plugin.filter.PlugInFilter;

public class VolumeFilter_ implements PlugInFilter
{
    protected   ImageStack stack;
    protected   StackStatistics istats;
    static boolean fNumber=true;
    static double  minvolume=67;
    static double  maxvolume=99999;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
    	{
            showAbout();
            return DONE;
        }
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        //----------------------------------------------------------------------
        /*Creates a dialog box, allowing the user to enter the required
        minimum and maximum volume.*/
        GenericDialog gd=new GenericDialog("Volume filter",IJ.getInstance());
        gd.addNumericField("Minimum volume:",minvolume,0,5,"pixels");
        gd.addNumericField("Maximum volume:",maxvolume,0,5,"pixels");
        gd.addCheckbox("Show number of remaining objects",fNumber);
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        minvolume=(double)gd.getNextNumber();
        maxvolume=(double)gd.getNextNumber();
        fNumber=gd.getNextBoolean();
        //----------------------------------------------------------------------
        stack=imp.getStack();
        StackStatistics stats=new StackStatistics (imp);
        istats=new StackStatistics(imp,(int)stats.histMax+1,0,stats.histMax);//Recalculates the histogram using integer beans
        return DOES_16+STACK_REQUIRED+NO_UNDO;
    }

    public void run(ImageProcessor ip)
    {
        boolean flLabel,relabel=false;
        int     i,j,z;
    	int     height=ip.getHeight();
        int     width=ip.getWidth();
        int     numRegions=(int)istats.histMax+1;
    	int[]	labels=new int[numRegions];//Table of labels
        int     minmodifiedlabel=numRegions;

        //Initialises table of labels
        for(i=1;i<numRegions;i++)
            labels[i]=i;
        //Verifies if there is at least one object smaller or bigger than the thresholds and eliminates them
        for (i=1;i<numRegions;i++)
            if (istats.histogram[i]<minvolume|istats.histogram[i]>maxvolume)
            {
                labels[i]=0;
                if (relabel==false)
                {
                    relabel=true;
                    minmodifiedlabel=i;//Identifies the smallest modified label
                }
            }
        //Minimises the region indexes in the list
        if (relabel)
        {
            for(i=minmodifiedlabel;i<numRegions;i++)
            {
                flLabel=true;
                for(j=i;j<numRegions;j++)//If a label is equal to its position on the list...
                    if(labels[j]==i)
                    {
                        flLabel=false;
                        break;
                    }
                while(flLabel)
                {
                    flLabel=false;
                    for(j=i;j<numRegions;j++)
                        if(labels[j]>i)
                        {
                            labels[j]--;
                            flLabel=true;
                        }
                    for(j=i;j<numRegions;j++)
                        if(labels[j]==i)
                        {
                            flLabel=false;
                            break;
                        }
                }
            }
            //Labels the regions in the resulting stack again
            for (z=0;z<stack.getSize();z++)
            {
                short[] pixels=(short[])((ShortProcessor)stack.getProcessor(z+1)).getPixels();
                for(i=0;i<height*width;i++)
                    pixels[i]=(short)labels[pixels[i]];
            }
        }
//------------------------------------------------------------------------------
//Counts the remaining objects
        if (fNumber)
        {
            if (relabel)
            {
                int max=0;
                for (i=1;i<numRegions;i++)
                    if (max<labels[i])
                        max=labels[i];
                IJ.log("Number of cells: "+max);
            }
            else
                IJ.log("Number of cells: "+--numRegions);
        }
    }

    void showAbout()
    {
        IJ.showMessage("About VolumeFilter_...",
        "This plug-in filter eliminates the 3D-objects that are smaller or bigger than a predefined volume.");
    }
}
